package net.shrine.protocol.version.v2

import net.shrine.protocol.version.v2.querydefinition.{QueryDefinition, XmlString}
import net.shrine.util.ShouldMatchersForJUnit
import org.junit.Test

class QueryTest extends ShouldMatchersForJUnit {

  val expectedQueryProgress: QueryProgress = V2JsonTest.staticQueryProgress
  val expectedQueryError: QueryError = V2JsonTest.staticQueryError

  @Test
  def testQueryAgainstV2Json():Unit = {
    V2JsonTest.testRoundTrip(expectedQueryProgress.asJsonText.underlying, expectedQueryProgress, Query.tryRead)
    V2JsonTest.testRoundTrip(expectedQueryError.asJsonText.underlying, expectedQueryError, Query.tryRead)
  }

  @Test
  def testQuerywithQdefXmlAndOuputXmlAgainstV2Json():Unit = {
    val staticQueryProgress: QueryProgress = Query.create(
      id = 123L,
      versionInfo = V2JsonTest.staticVersionInfo,
      queryDefinition = QueryDefinition(XmlString(
        """
          |<query_definition>
          |<query_name>GenderNewAgainXml@12:58:04</query_name>
          |<query_timing>ANY</query_timing>
          |<specificity_scale>0</specificity_scale>
          |<use_shrine>1</use_shrine>
          |<panel>
          |	<panel_number>1</panel_number>
          |	<panel_accuracy_scale>100</panel_accuracy_scale>
          |	<invert>0</invert>
          |	<panel_timing>ANY</panel_timing>
          |	<total_item_occurrences>1</total_item_occurrences>
          |	<item>
          |		<hlevel>3</hlevel>
          |		<item_name>Gender</item_name>
          |		<item_key>\\\\\\\\i2b2_DEMO\\\\i2b2\\\\Demographics\\\\Gender\\\\</item_key>
          |		<tooltip>\\\\\\\\i2b2_DEMO\\\\i2b2\\\\Demographics\\\\Gender\\\\</tooltip>
          |		<class>ENC</class>
          |		<constrain_by_date></constrain_by_date>
          |		<item_icon>LA</item_icon>
          |		<item_is_synonym>false</item_is_synonym>
          |	</item>
          |</panel>undefined</query_definition>\",
          |\"encodedClass\":\"XmlString
          |""".stripMargin)),
      breakdownNames = Seq.empty,
      queryName = "queryName",
      queryNotes = None,
      queryFaved = false,
      nodeOfOriginId = V2JsonTest.staticNodeId,
      researcherId = V2JsonTest.staticResearcherId,
    )
    V2JsonTest.testRoundTrip(staticQueryProgress.asJsonText.underlying, staticQueryProgress, Query.tryRead)
  }
}
