import React from "react";
import PropTypes from "prop-types";

import { TableCell, TableRow, Typography } from "@material-ui/core";
import { InstitutionResult } from "models";
import { getCount } from "utilities";
import { Status } from "./Status";
import { ClickableStatus } from "./ClickableStatus";

export const InstitutionResultStatus = ({ result }) => {
  const { internalStatus, status } = result;

  let resultType;
  switch (internalStatus.toUpperCase()) {
    case "RESULT FROM CRC":
    case "FINISHED":
      resultType = (
        <Status statusText={getCount(result.count)} className="PatientCount" />
      );
      break;
    case "ID ASSIGNED":
    case "SENT TO ADAPTER":
    case "RECEIVED BY ADAPTER":
    case "READY TO SUBMIT":
    case "SUBMITTED TO CRC":
      resultType = <Status statusText={status} className="ProcessingAtSite" />;
      break;
    case "ERROR IN SHRINE":
    case "UKNOWN WHILE QUEUED BY CRC":
    case "ERROR FROM CRC":
    case "UNKNOWN FINAL":
    case "ERROR":
    case "PROCESSING":
    case "RUNNING":
    case "QUEUED":
    case "INCOMPLETE":
    case "HELD":
    case "SMALL_QUEUE":
    case "TIMEDOUT":
    case "MEDIUM_QUEUE":
    case "LARGE_QUEUE":
    case "NO_MORE_QUEUE":
    case "MEDIUM_QUEUE_RUNNING":
    case "LARGE_QUEUE_RUNNING":
    case "HUB_WILL_SUBMI":
      resultType = (
        <ClickableStatus
          statusText={status}
          className="SiteError"
          data={result.errorDetailInfo}
        />
      );
      break;
    default:
      resultType = <Status statusText={status} className="DelayedAtSite" />;
  }

  return (
    <TableRow>
      <TableCell>
        <Typography>{result.institutionName}</Typography>
      </TableCell>
      <TableCell>
        <Typography>{resultType}</Typography>
      </TableCell>
    </TableRow>
  );
};

InstitutionResultStatus.propTypes = {
  result: PropTypes.shape(InstitutionResult.propTypes).isRequired
};
